@@ -80,10 +80,13 @@ class DummyRados(object):
         self.cmd_names['fs ls'] = '''{"format": "json", "prefix": "fs ls"}'''
         self.cmd_names['quorum_status'] = '''{"format": "json", "prefix": "quorum_status"}'''
         self.cmd_names['caps_change_default_pool_prefix'] = '''{"caps": ["mon", "allow r, allow command quorum_status, allow command version", "mgr", "allow command config", "osd", "allow rwx pool=default.rgw.meta, allow r pool=.rgw.root, allow rw pool=default.rgw.control, allow rx pool=default.rgw.log, allow x pool=default.rgw.buckets.index"], "entity": "client.healthchecker", "format": "json", "prefix": "auth caps"}'''
+        self.cmd_names['mgr services'] = '''{"format": "json", "prefix": "mgr services"}'''
         # all the commands and their output
         self.cmd_output_map[self.cmd_names['fs ls']
                             ] = '''[{"name":"myfs","metadata_pool":"myfs-metadata","metadata_pool_id":2,"data_pool_ids":[3],"data_pools":["myfs-data0"]}]'''
         self.cmd_output_map[self.cmd_names['quorum_status']] = '''{"election_epoch":3,"quorum":[0],"quorum_names":["a"],"quorum_leader_name":"a","quorum_age":14385,"features":{"quorum_con":"4540138292836696063","quorum_mon":["kraken","luminous","mimic","osdmap-prune","nautilus","octopus"]},"monmap":{"epoch":1,"fsid":"af4e1673-0b72-402d-990a-22d2919d0f1c","modified":"2020-05-07T03:36:39.918035Z","created":"2020-05-07T03:36:39.918035Z","min_mon_release":15,"min_mon_release_name":"octopus","features":{"persistent":["kraken","luminous","mimic","osdmap-prune","nautilus","octopus"],"optional":[]},"mons":[{"rank":0,"name":"a","public_addrs":{"addrvec":[{"type":"v2","addr":"10.110.205.174:3300","nonce":0},{"type":"v1","addr":"10.110.205.174:6789","nonce":0}]},"addr":"10.110.205.174:6789/0","public_addr":"10.110.205.174:6789/0","priority":0,"weight":0}]}}'''
+        self.cmd_output_map[self.cmd_names['mgr services']
+                            ] = '''{"dashboard":"https://ceph-dashboard:8443/","prometheus":"http://ceph-dashboard-db:9283/"}'''
         self.cmd_output_map['''{"caps": ["mon", "allow r, allow command quorum_status", "osd", "allow rwx pool=default.rgw.meta, allow r pool=.rgw.root, allow rw pool=default.rgw.control, allow x pool=default.rgw.buckets.index"], "entity": "client.healthchecker", "format": "json", "prefix": "auth get-or-create"}'''] = '''[{"entity":"client.healthchecker","key":"AQDFkbNeft5bFRAATndLNUSEKruozxiZi3lrdA==","caps":{"mon":"allow r, allow command quorum_status","osd":"allow rwx pool=default.rgw.meta, allow r pool=.rgw.root, allow rw pool=default.rgw.control, allow x pool=default.rgw.buckets.index"}}]'''
         self.cmd_output_map['''{"caps": ["mon", "profile rbd", "osd", "profile rbd"], "entity": "client.csi-rbd-node", "format": "json", "prefix": "auth get-or-create"}'''] = '''[{"entity":"client.csi-rbd-node","key":"AQBOgrNeHbK1AxAAubYBeV8S1U/GPzq5SVeq6g==","caps":{"mon":"profile rbd","osd":"profile rbd"}}]'''
         self.cmd_output_map['''{"caps": ["mon", "profile rbd", "mgr", "allow rw", "osd", "profile rbd"], "entity": "client.csi-rbd-provisioner", "format": "json", "prefix": "auth get-or-create"}'''] = '''[{"entity":"client.csi-rbd-provisioner","key":"AQBNgrNe1geyKxAA8ekViRdE+hss5OweYBkwNg==","caps":{"mgr":"allow rw","mon":"profile rbd","osd":"profile rbd"}}]'''
@@ -93,12 +96,6 @@ class DummyRados(object):
         self.cmd_output_map['''{"format": "json", "prefix": "mgr services"}'''] = '''{"dashboard": "http://rook-ceph-mgr-a-57cf9f84bc-f4jnl:7000/", "prometheus": "http://rook-ceph-mgr-a-57cf9f84bc-f4jnl:9283/"}'''
         self.cmd_output_map['''{"entity": "client.healthchecker", "format": "json", "prefix": "auth get"}'''] = '''{"dashboard": "http://rook-ceph-mgr-a-57cf9f84bc-f4jnl:7000/", "prometheus": "http://rook-ceph-mgr-a-57cf9f84bc-f4jnl:9283/"}'''
         self.cmd_output_map['''{"entity": "client.healthchecker", "format": "json", "prefix": "auth get"}'''] = '''[{"entity":"client.healthchecker","key":"AQDFkbNeft5bFRAATndLNUSEKruozxiZi3lrdA==","caps":{"mon": "allow r, allow command quorum_status, allow command version", "mgr": "allow command config", "osd": "allow rwx pool=default.rgw.meta, allow r pool=.rgw.root, allow rw pool=default.rgw.control, allow rx pool=default.rgw.log, allow x pool=default.rgw.buckets.index"}}]'''
-        self.cmd_output_map['''{"entity": "client.csi-cephfs-node", "format": "json", "prefix": "auth get"}'''] = '''[]'''
-        self.cmd_output_map['''{"entity": "client.csi-rbd-node", "format": "json", "prefix": "auth get"}'''] = '''[]'''
-        self.cmd_output_map['''{"entity": "client.csi-rbd-provisioner", "format": "json", "prefix": "auth get"}'''] = '''[]'''
-        self.cmd_output_map['''{"entity": "client.csi-cephfs-provisioner", "format": "json", "prefix": "auth get"}'''] = '''[]'''
-        self.cmd_output_map['''{"entity": "client.csi-cephfs-provisioner-openshift-storage", "format": "json", "prefix": "auth get"}'''] = '''[]'''
-        self.cmd_output_map['''{"entity": "client.csi-cephfs-provisioner-openshift-storage-myfs", "format": "json", "prefix": "auth get"}'''] = '''[]'''
         self.cmd_output_map[self.cmd_names['caps_change_default_pool_prefix']] = '''[{}]'''
         self.cmd_output_map['{"format": "json", "prefix": "status"}'] = ceph_status_str
 
@@ -182,6 +179,10 @@ class RadosJSON:
                                   help="Provides the name of the RBD datapool")
         output_group.add_argument("--rgw-endpoint", default="", required=False,
                                   help="Rados GateWay endpoint (in <IP>:<PORT> format)")
+        output_group.add_argument("--rgw-tls-cert-path", default="", required=False,
+                                  help="Rados GateWay endpoint TLS certificate")
+        output_group.add_argument("--rgw-skip-tls", required=False, default=False,
+                                  help="Ignore TLS certification validation when a self-signed certificate is provided (NOT RECOMMENDED")
         output_group.add_argument("--monitoring-endpoint", default="", required=False,
                                   help="Ceph Manager prometheus exporter endpoints (comma separated list of <IP> entries of active and standby mgrs)")
         output_group.add_argument("--monitoring-endpoint-port", default="", required=False,
@@ -198,6 +199,12 @@ class RadosJSON:
             args_to_parse = sys.argv[1:]
         return argP.parse_args(args_to_parse)
 
+    def validate_rgw_endpoint_tls_cert(self):
+        if self._arg_parser.rgw_tls_cert_path:
+            with open(self._arg_parser.rgw_tls_cert_path, encoding='utf8') as f:
+                contents = f.read()
+                return contents.rstrip()
+
     def _check_conflicting_options(self):
         if not self._arg_parser.upgrade and not self._arg_parser.rbd_data_pool_name:
             raise ExecutionFailureException(
@@ -236,7 +243,7 @@ class RadosJSON:
                 "Out of range port number: {}".format(port))
         return False
 
-    def endpoint_dial(self, endpoint_str, timeout=3):
+    def endpoint_dial(self, endpoint_str, timeout=3, cert=None):
         # if the 'cluster' instance is a dummy one,
         # don't try to reach out to the endpoint
         if isinstance(self.cluster, DummyRados):
@@ -245,7 +252,14 @@ class RadosJSON:
         for prefix in protocols:
             try:
                 ep = "{}://{}".format(prefix, endpoint_str)
-                r = requests.head(ep, timeout=timeout)
+                # If verify is set to a path to a directory,
+                # the directory must have been processed using the c_rehash utility supplied with OpenSSL.
+                if prefix == "https" and cert and self._arg_parser.rgw_skip_tls:
+                    r = requests.head(ep, timeout=timeout, verify=False)
+                elif prefix == "https" and cert:
+                    r = requests.head(ep, timeout=timeout, verify=cert)
+                else:
+                    r = requests.head(ep, timeout=timeout)
                 if r.status_code == 200:
                     return
             except:
@@ -356,9 +370,9 @@ class RadosJSON:
 
     def get_active_and_standby_mgrs(self):
         monitoring_endpoint_port = self._arg_parser.monitoring_endpoint_port
-        monitoring_endpoint_ip = self._arg_parser.monitoring_endpoint
+        monitoring_endpoint_ip_list = self._arg_parser.monitoring_endpoint
         standby_mgrs = []
-        if not monitoring_endpoint_ip:
+        if not monitoring_endpoint_ip_list:
             cmd_json = {"prefix": "status", "format": "json"}
             ret_val, json_out, err_msg = self._common_cmd_json_gen(cmd_json)
             # if there is an unsuccessful attempt,
@@ -381,7 +395,7 @@ class RadosJSON:
             except ValueError:
                 raise ExecutionFailureException(
                     "invalid endpoint: {}".format(monitoring_endpoint))
-            monitoring_endpoint_ip = parsed_endpoint.hostname
+            monitoring_endpoint_ip_list = parsed_endpoint.hostname
             if not monitoring_endpoint_port:
                 monitoring_endpoint_port = "{}".format(parsed_endpoint.port)
 
@@ -389,6 +403,18 @@ class RadosJSON:
         if not monitoring_endpoint_port:
             monitoring_endpoint_port = self.DEFAULT_MONITORING_ENDPOINT_PORT
 
+        # user could give comma and space separated inputs (like --monitoring-endpoint="<ip1>, <ip2>")
+        monitoring_endpoint_ip_list = monitoring_endpoint_ip_list.replace(
+            ",", " ")
+        monitoring_endpoint_ip_list_split = monitoring_endpoint_ip_list.split()
+        # if monitoring-endpoint could not be found, raise an error
+        if len(monitoring_endpoint_ip_list_split) == 0:
+            raise ExecutionFailureException("No 'monitoring-endpoint' found")
+        # first ip is treated as the main monitoring-endpoint
+        monitoring_endpoint_ip = monitoring_endpoint_ip_list_split[0]
+        # rest of the ip-s are added to the 'standby_mgrs' list
+        standby_mgrs.extend(monitoring_endpoint_ip_list_split[1:])
+
         try:
             failed_ip = monitoring_endpoint_ip
             monitoring_endpoint_ip = self._convert_hostname_to_ip(
@@ -413,14 +439,6 @@ class RadosJSON:
         all_mgr_ips_str = ",".join(mgr_ips)
         return all_mgr_ips_str, monitoring_endpoint_port
 
-    def check_user_exist(self,user):
-        cmd_json = {"prefix": "auth get", "entity": "{}".format(
-            user), "format": "json"}
-        ret_val, json_out, _ = self._common_cmd_json_gen(cmd_json)
-        if ret_val != 0 or len(json_out) == 0:
-            return ""
-        return str(json_out[0]['key'])
-    
     def create_cephCSIKeyring_cephFSProvisioner(self):
         '''
         command: ceph auth get-or-create client.csi-cephfs-provisioner mon 'allow r' mgr 'allow rw' osd 'allow rw tag cephfs metadata=*'
@@ -430,11 +448,6 @@ class RadosJSON:
                     "caps": ["mon", "allow r", "mgr", "allow rw",
                              "osd", "allow rw tag cephfs metadata=*"],
                     "format": "json"}
-        # check if user already exist
-        user_key = self.check_user_exist("client.csi-cephfs-provisioner")
-        if user_key != "":
-            return user_key
-        
         ret_val, json_out, err_msg = self._common_cmd_json_gen(cmd_json)
         # if there is an unsuccessful attempt,
         if ret_val != 0 or len(json_out) == 0:
@@ -451,11 +464,6 @@ class RadosJSON:
                              "osd", "allow rw tag cephfs *=*",
                              "mds", "allow rw"],
                     "format": "json"}
-        # check if user already exist
-        user_key = self.check_user_exist("client.csi-cephfs-node")
-        if user_key != "":
-            return user_key
- 
         ret_val, json_out, err_msg = self._common_cmd_json_gen(cmd_json)
         # if there is an unsuccessful attempt,
         if ret_val != 0 or len(json_out) == 0:
@@ -471,11 +479,6 @@ class RadosJSON:
                              "mgr", "allow rw",
                              "osd", "profile rbd"],
                     "format": "json"}
-        # check if user already exist
-        user_key = self.check_user_exist("client.csi-rbd-provisioner")
-        if user_key != "":
-            return user_key
-        
         ret_val, json_out, err_msg = self._common_cmd_json_gen(cmd_json)
         # if there is an unsuccessful attempt,
         if ret_val != 0 or len(json_out) == 0:
@@ -575,11 +578,6 @@ class RadosJSON:
                     "caps": ["mon", "profile rbd",
                              "osd", "profile rbd"],
                     "format": "json"}
-        # check if user already exist
-        user_key = self.check_user_exist("client.csi-rbd-node")
-        if user_key != "":
-            return user_key
-        
         ret_val, json_out, err_msg = self._common_cmd_json_gen(cmd_json)
         # if there is an unsuccessful attempt,
         if ret_val != 0 or len(json_out) == 0:
@@ -603,6 +601,16 @@ class RadosJSON:
                 "Error: {}".format(err_msg if ret_val != 0 else self.EMPTY_OUTPUT_LIST))
         return str(json_out[0]['key'])
 
+    def get_ceph_dashboard_link(self):
+        cmd_json = {"prefix": "mgr services", "format": "json"}
+        ret_val, json_out, _ = self._common_cmd_json_gen(cmd_json)
+        # if there is an unsuccessful attempt,
+        if ret_val != 0 or len(json_out) == 0:
+            return None
+        if not 'dashboard' in json_out:
+            return None
+        return json_out['dashboard']
+
     def create_rgw_admin_ops_user(self):
         cmd = ['radosgw-admin', 'user', 'create', '--uid', self.EXTERNAL_RGW_ADMIN_OPS_USER_NAME, '--display-name',
                'Rook RGW Admin Ops user', '--caps', 'buckets=*;users=*;usage=read;metadata=read;zone=read']
@@ -637,7 +645,8 @@ class RadosJSON:
         # if rgw_endpoint is provided, validate it
         if self._arg_parser.rgw_endpoint:
             self._invalid_endpoint(self._arg_parser.rgw_endpoint)
-            self.endpoint_dial(self._arg_parser.rgw_endpoint)
+            self.endpoint_dial(self._arg_parser.rgw_endpoint,
+                               cert=self.validate_rgw_endpoint_tls_cert())
             rgw_pool_to_validate = ["{0}.rgw.meta".format(self._arg_parser.rgw_pool_prefix),
                                     ".rgw.root",
                                     "{0}.rgw.control".format(
@@ -645,6 +654,7 @@ class RadosJSON:
                 "{0}.rgw.log".format(
                 self._arg_parser.rgw_pool_prefix)]
             pools_to_validate.extend(rgw_pool_to_validate)
+
         for pool in pools_to_validate:
             if not self.cluster.pool_exists(pool):
                 raise ExecutionFailureException(
@@ -657,6 +667,7 @@ class RadosJSON:
         self.out_map['ROOK_EXTERNAL_USERNAME'] = self.run_as_user
         self.out_map['ROOK_EXTERNAL_CEPH_MON_DATA'] = self.get_ceph_external_mon_data()
         self.out_map['ROOK_EXTERNAL_USER_SECRET'] = self.create_checkerKey()
+        self.out_map['ROOK_EXTERNAL_DASHBOARD_LINK'] = self.get_ceph_dashboard_link()
         self.out_map['CSI_RBD_NODE_SECRET_SECRET'] = self.create_cephCSIKeyring_RBDNode()
         self.out_map['CSI_RBD_PROVISIONER_SECRET'] = self.create_cephCSIKeyring_RBDProvisioner()
         self.out_map['CEPHFS_POOL_NAME'] = self._arg_parser.cephfs_data_pool_name
@@ -669,12 +680,15 @@ class RadosJSON:
             )
             self.out_map['CSI_CEPHFS_PROVISIONER_SECRET'] = self.create_cephCSIKeyring_cephFSProvisioner()
         self.out_map['RGW_ENDPOINT'] = self._arg_parser.rgw_endpoint
+        self.out_map['RGW_TLS_CERT'] = ''
         self.out_map['MONITORING_ENDPOINT'], \
             self.out_map['MONITORING_ENDPOINT_PORT'] = self.get_active_and_standby_mgrs()
         self.out_map['RBD_POOL_NAME'] = self._arg_parser.rbd_data_pool_name
         self.out_map['RGW_POOL_PREFIX'] = self._arg_parser.rgw_pool_prefix
         if self._arg_parser.rgw_endpoint:
             self.out_map['ACCESS_KEY'], self.out_map['SECRET_KEY'] = self.create_rgw_admin_ops_user()
+            if self._arg_parser.rgw_tls_cert_path:
+                self.out_map['RGW_TLS_CERT'] = self.validate_rgw_endpoint_tls_cert()
 
     def gen_shell_out(self):
         self._gen_output_map()
@@ -740,6 +754,16 @@ class RadosJSON:
             }
         ]
 
+        # if 'ROOK_EXTERNAL_DASHBOARD_LINK' exists, then only add 'rook-ceph-dashboard-link' Secret
+        if self.out_map['ROOK_EXTERNAL_DASHBOARD_LINK']:
+            json_out.append({
+                "name": "rook-ceph-dashboard-link",
+                "kind": "Secret",
+                "data": {
+                    "userID": 'ceph-dashboard-link',
+                    "userKey": self.out_map['ROOK_EXTERNAL_DASHBOARD_LINK']
+                }
+            })
         # if 'CSI_RBD_PROVISIONER_SECRET' exists, then only add 'rook-csi-rbd-provisioner' Secret
         if self.out_map['CSI_RBD_PROVISIONER_SECRET']:
             json_out.append({
@@ -799,6 +823,15 @@ class RadosJSON:
                         "secretKey": self.out_map['SECRET_KEY']
                     }
                 })
+        # if 'RGW_TLS_CERT' exists, then only add the "ceph-rgw-tls-cert" secret
+        if self.out_map['RGW_TLS_CERT']:
+            json_out.append({
+                "name": "ceph-rgw-tls-cert",
+                "kind": "Secret",
+                "data": {
+                    "cert": self.out_map['RGW_TLS_CERT'],
+                }
+            })
         return json.dumps(json_out)+LINESEP
 
     def upgrade_user_permissions(self):
